// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_REACTOR_H
#define OB_REACTOR_H

namespace OB
{

//
// Event handler handle
// The event mask
// The event types
//
typedef int Handle;
typedef CORBA::ULong Mask;
const Mask EventRead = 0x1;
const Mask EventWrite = 0x2;

//
// The EventHandler class
//
class EventHandler
{
    //
    // Hide copy-constructor and assignment operator
    //
    EventHandler(const EventHandler&);
    void operator=(const EventHandler&);

protected:

    EventHandler() { }
    virtual ~EventHandler() { }

public:

    //
    // Handle event
    //
    virtual void handleEvent(Mask) = 0;
};

//
// The Reactor base class
//
class Reactor
{
    //
    // Hide copy-constructor and assignment operator
    //
    Reactor(const Reactor&);
    void operator=(const Reactor&);

protected:

    //
    // The Reactor singleton
    //
    static OB_IMPORT Reactor* instance_;
    friend class ReactorDestroyer;

    Reactor() { }
    virtual ~Reactor() { }

public:

    //
    // Get the reactor singleton
    //
    static Reactor* instance();

    //
    // Register and unregister event handler
    //
    virtual void registerHandler(EventHandler*, Mask, Handle) = 0;
    virtual void unregisterHandler(EventHandler*) = 0;

    //
    // Dispatch events
    //
    virtual void dispatch() = 0;
    virtual bool dispatchOneEvent(CORBA::Long) = 0;

    //
    // Test to see if an event is available
    //
    virtual bool eventReady() = 0;

    //
    // Stop event dispatching
    //
    virtual void stop() = 0;
};

} // End of namespace OB

#endif
